import { Storage } from './Storage';
export declare class LocalStorage extends Storage {
    /**
     * Get a value from the local storage.
     * @param key - The storage key.
     * @param defaultValue - The default value to return if the key is not found.
     * @returns the value from the local storage.
     * @throws if the local storage is not supported.
     * @example
     * ```typescript
     * const storage = new LocalStorage('oauth')
     * storage.set('test', 'value')
     * expect(storage.get('test')).toBe('value')
     * ```
     */
    get(key: string, defaultValue?: string): string | undefined;
    /**
     * Set a value in the local storage.
     * @param key - The storage key.
     * @param value - The value to store.
     * @throws if the local storage is not supported.
     * @example
     * ```typescript
     * const storage = new LocalStorage('oauth')
     * storage.set('test', 'value')
     * expect(storage.get('test')).toBe('value')
     * ```
     */
    set(key: string, value?: string | null): void;
    /**
     * Delete a value from the local storage.
     * @param name - The storage key.
     * @throws if the local storage is not supported.
     * @example
     * ```typescript
     * const storage = new LocalStorage('oauth')
     * storage.set('test', 'value')
     * expect(storage.get('test')).toBe('value')
     * storage.delete('test')
     * expect(storage.get('test')).toBeNull()
     * ```
     */
    delete(name: string): void;
    /**
     * Clear all values from the local storage.
     * @throws if the local storage is not supported.
     * @example
     * ```typescript
     * const storage = new LocalStorage('oauth')
     * storage.set('test', 'value')
     * expect(storage.get('test')).toBe('value')
     * storage.clear()
     * expect(storage.get('test')).toBeNull()
     * ```
     */
    clear(): void;
    /**
     * Check if the local storage is supported.
     * @returns true if the local storage is supported.
     * @example
     * ```typescript
     * expect(LocalStorage.supported()).toBe(true)
     * ```
     */
    static supported(): boolean;
    private _checkSupport;
}
