import { Storage as o } from "./Storage.js";
class s extends o {
  /**
   * Get a value from the session storage.
   * @param name - The storage key.
   * @param defaultValue - The default value to return if the key is not found.
   * @returns the value from the session storage.
   * @throws if the session storage is not supported.
   * @example
   * ```typescript
   * const storage = new SessionStorage('oauth')
   * storage.set('test', 'value')
   * expect(storage.get('test')).toBe('value')
   * ```
   */
  get(e, t) {
    return this._checkSupport(), sessionStorage.getItem(this.key(e)) ?? t;
  }
  /**
   * Set a value in the session storage.
   * @param name - The storage key.
   * @param value - The value to store.
   * @throws if the session storage is not supported.
   * @example
   * ```typescript
   * const storage = new SessionStorage('oauth')
   * storage.set('test', 'value')
   * expect(storage.get('test')).toBe('value')
   * ```
   */
  set(e, t) {
    if (this._checkSupport(), t == null) {
      this.delete(e);
      return;
    }
    sessionStorage.setItem(this.key(e), t);
  }
  /**
   * Delete a value from the session storage.
   * @param key - The storage key.
   * @throws if the session storage is not supported.
   * @example
   * ```typescript
   * const storage = new SessionStorage('oauth')
   * storage.set('test', 'value')
   * expect(storage.get('test')).toBe('value')
   * storage.delete('test')
   * expect(storage.get('test')).toBeNull()
   * ```
   */
  delete(e) {
    this._checkSupport(), sessionStorage.removeItem(this.key(e));
  }
  /**
   * Clear all values from the session storage.
   * @throws if the session storage is not supported.
   * @example
   * ```typescript
   * const storage = new SessionStorage('oauth')
   * storage.set('test', 'value')
   * expect(storage.get('test')).toBe('value')
   * storage.clear()
   * expect(storage.get('test')).toBeNull()
   * ```
   */
  clear() {
    this._checkSupport();
    const e = this.key();
    for (const t in sessionStorage)
      t.indexOf(e) === 0 && this.delete(t);
  }
  /**
   * Determines if the current browser supports session storage.
   * @returns true if the session storage is supported.
   * @example
   * ```typescript
   * expect(SessionStorage.supported()).toBe(true)
   * ```
   */
  static supported() {
    return typeof window < "u" && typeof sessionStorage < "u";
  }
  _checkSupport() {
    if (!s.supported())
      throw new Error("Session storage is not supported");
  }
}
export {
  s as SessionStorage
};
